package com.example.nto.aspect;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;

import java.util.Arrays;

@Aspect
@Component
public class LoggingAspect {
    // Это класс логирования. Здесь находятся методы, отвечающие за вывод дополнительной информации в консоль.

    // Чтобы понять, что здесь происходит надо посмотреть >>>
    // > https://disk.yandex.ru/i/E34JfcAK8nA4Vw
    // > https://disk.yandex.ru/i/9TNQ8D5V3cPjrg

    // Главный класс, поставляется вместе с `spring-boot-starter-web`.
    // ! Библиотека называется log4j !
    private static final Logger logger = LogManager.getLogger(LoggingAspect.class);

    // Аннотация, объединяющая @Before и @AfterReturning.
    // Тут мы говорим, какая аннотация будет запускать процесс логирования в методе.
    // Сюда также можно передать путь к классу, чтобы все методы из него логировались, но делать так не надо.
    //                    ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    @Around("@annotation (com.example.nto.aspect.annotation.LogExample)")
    // Это что-то типа представления метода, тут хранятся название, аргументы, возвращаемое значение и тд.
    //                      ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    public Object logAround(ProceedingJoinPoint joinPoint) throws Throwable {
        logger.info(
                "  >>> Логирование метода: {} | Входные параметры: {} <<<",
                joinPoint.getSignature().getName(),  // Получение названия метода.
                Arrays.toString(joinPoint.getArgs()) // Получение аргументов, передаваемы в метод.
        );

        Object result = joinPoint.proceed();  // Получает результат выполнения функции.

        logger.info("  >>> Завершил работу метод: {} | Результат работы: {} <<<",
                joinPoint.getSignature().getName(), result);
        return result;
    }

    //                            Эта штука должна иметь такое же название, как и аргумент `Object` снизу.
    //                                                                                  ↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓↓
    @AfterThrowing(pointcut = "@annotation (com.example.nto.aspect.annotation.LogExample)", throwing = "error")
    public void logAfterThrowing(JoinPoint joinPoint, Object error) {
        logger.error(error.toString());
    }
}
